@@ -25,6 +25,7 @@
 
 #define PHOC_LAYER_SHELL_EFFECTS_PROTOCOL_NAME "zphoc_layer_shell_effects_v1"
 #define PHOSH_PRIVATE_PROTOCOL_NAME "phosh_private"
+#define PHOSH_MOBILE_SETTINGS_DESCRIPTION _("- Manage your mobile settings")
 
 enum {
   PROP_0,
@@ -54,10 +55,20 @@ struct _MobileSettingsApplication {
 G_DEFINE_TYPE (MobileSettingsApplication, mobile_settings_application, ADW_TYPE_APPLICATION)
 
 static const GOptionEntry entries[] = {
+  {
+    "version", 'v',
+    G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE,
+    NULL, "Current version of phosh-mobile-settings", NULL
+  },
+  {
+    "debug-info", 'd',
+    G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE,
+    NULL, "Print debugging information", NULL
+  },
   {
     "list", 'l',
     G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE,
-    NULL, "Lists the available panels in phosh-mobile-settings", NULL
+    NULL, "List the available panels", NULL
   },
   {
     G_OPTION_REMAINING, '\0',
@@ -138,6 +149,24 @@ static const struct wl_registry_listener registry_listener = {
 };
 
 
+static void
+setup_wayland (MobileSettingsApplication *self)
+{
+  GdkDisplay *gdk_display;
+
+  if (self->wl_display == NULL) {
+    gdk_display = gdk_display_get_default ();
+    self->wl_display = gdk_wayland_display_get_wl_display (gdk_display);
+    if (self->wl_display != NULL) {
+      self->wl_registry = wl_display_get_registry (self->wl_display);
+      wl_registry_add_listener (self->wl_registry, &registry_listener, self);
+    } else {
+      g_critical ("Failed to get display: %m\n");
+    }
+  }
+}
+
+
 static GtkWindow *
 get_active_window (MobileSettingsApplication *self)
 {
@@ -150,6 +179,31 @@ get_active_window (MobileSettingsApplication *self)
 }
 
 
+static void
+print_version (void)
+{
+  g_print ("Phosh Mobile Settings %s %s\n",
+           MOBILE_SETTINGS_VERSION,
+           PHOSH_MOBILE_SETTINGS_DESCRIPTION);
+}
+
+
+static void
+print_system_information (GApplication *app)
+{
+  MobileSettingsApplication *self = MOBILE_SETTINGS_APPLICATION (app);
+
+  /* We need this to init components and generate debug info */
+  adw_init ();
+
+  setup_wayland (self);
+  /* Make sure all pending events got processed by the server */
+  wl_display_roundtrip (self->wl_display);
+
+  g_print ("Debugging information:\n%s", mobile_settings_generate_debug_info ());
+}
+
+
 static void
 list_available_panels (GApplication *app)
 {
@@ -214,7 +268,15 @@ mobile_settings_application_handle_local_options (GApplication *app,
   g_autofree GStrv panels = NULL;
   GApplicationClass *app_class = G_APPLICATION_CLASS (mobile_settings_application_parent_class);
 
-  if (g_variant_dict_contains (options, "list")) {
+  if (g_variant_dict_contains (options, "version")) {
+    print_version ();
+
+    return 0;
+  } else if (g_variant_dict_contains (options, "debug-info")) {
+    print_system_information (app);
+
+    return 0;
+  } else if (g_variant_dict_contains (options, "list")) {
     list_available_panels (app);
 
     return 0;
@@ -236,23 +298,13 @@ static void
 mobile_settings_application_activate (GApplication *app)
 {
   GtkWindow *window;
-  GdkDisplay *gdk_display;
   MobileSettingsApplication *self = MOBILE_SETTINGS_APPLICATION (app);
 
   g_assert (GTK_IS_APPLICATION (app));
 
   window = get_active_window (self);
 
-  if (self->wl_display == NULL) {
-    gdk_display = gdk_display_get_default ();
-    self->wl_display = gdk_wayland_display_get_wl_display (gdk_display);
-    if (self->wl_display != NULL) {
-      self->wl_registry = wl_display_get_registry (self->wl_display);
-      wl_registry_add_listener (self->wl_registry, &registry_listener, self);
-    } else {
-      g_critical ("Failed to get display: %m\n");
-    }
-  }
+  setup_wayland (self);
 
   gtk_window_present (window);
 }
@@ -364,7 +416,7 @@ mobile_settings_application_init (MobileSettingsApplication *self)
   });
 
   g_application_set_option_context_parameter_string (G_APPLICATION (self),
-                                                     _("- Manage your mobile settings"));
+                                                     PHOSH_MOBILE_SETTINGS_DESCRIPTION);
   g_application_add_main_option_entries (G_APPLICATION (self), entries);
 
   self->device_plugin_loader = ms_plugin_loader_new (plugin_dirs, MS_EXTENSION_POINT_DEVICE_PANEL);
@@ -413,8 +465,7 @@ mobile_settings_application_get_wayland_protocols (MobileSettingsApplication *se
   g_assert (MOBILE_SETTINGS_APPLICATION (self));
 
   keys = g_hash_table_get_keys (self->wayland_protocols);
-  if (keys == NULL)
-    return NULL;
+  g_assert (keys);
 
   for (GList *l = keys; l; l = l->next)
     g_strv_builder_add (protocols, l->data);
